<?php
/*
//================================================================================
* Novahq.net Heartbeat DLL PHP Status Script *
//================================================================================
:- First Build: August 22nd 2017
:- Last Build: August 30th 2017
:- Author: Scott Lucht <scott@novahq.net> http://www.novahq.net
:- Compatibility: DF1 / DF2 / DFLW / DFTFD / DFBHD+TS / TIOCoop (Windows XP to Windows 10)
:- Copyright (c) 2017 All Rights Reserved
:-
:- This program is distributed in the hope that it will be useful,
:- but WITHOUT ANY WARRANTY; without even the implied warranty of
:- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
:-
:- PUBLIC RELEASE: You may share or make modifications to this file
:- as long as this Readme.txt file is included with your distribution
:- 
:- PRIVATE RELEASE: Contact me for info
:-
//================================================================================
* Description
//================================================================================
:- NovaHQ Heartbeat DLL sends your games information to alternate game lobbies
:- such as nw.novahq.net and novaworld.cc. This DLL also has the ability to send
:- game status information to your own website. This PHP script is a full example
:- on how to use that feature. You may distribute your own version of the *.ini
:- file(s) that are included with the Heartbeat DLL download that has your own
:- lobby URL  already added. The Heartbeat DLL was created to ensure that these 
:- classic games can live on forever even without NovaLogic or THQ Nordic.
:- This PHP Script is only compatible with the Heartbeat DLL >= 1.0.5
//================================================================================
* Installation
//================================================================================
:- Upload the PHP files to a location on your website. Setup your MySQL database
:- in your control panel and record the username and password. Open the config.php
:- file and enter your database information and adjust a few of the variables to
:- your liking. Navigate to the folder you uploaded the files to on your website,
:- and run the install.php file in your browser. Once that is done, DELETE the 
:- install.php file and you're ready to go.
//================================================================================
* Bug Reports
//================================================================================
:- If you have any issues with this script, please post in the Novahq.net Forums
:- This script was tested with PHP 5.6, PHP 7.0, 7.1 and 7.2 beta.
//================================================================================
* Change log
//================================================================================
:- Version 1.0.0
:-		1) Initial Release (Compatible with Heartbeat DLL >= 1.0.5)
:-
:- Version 1.0.1
:-		1) Fixed direct access to global.php issue
:-		2) Add version to debug
*/

require_once("./config.php");
require_once("./global.php");

if($Debug_Mode && $_SERVER['REQUEST_METHOD'] == 'POST' && !empty($_POST))
	ExitDebug(print_r($_POST, true).PHP_EOL, false);

//########################################################################
// Check Submission
//########################################################################

if($_SERVER['REQUEST_METHOD'] != 'POST')
	ExitDebug("Invalid Method");

if(!isset($_POST['SKey']))
	ExitDebug("Missing Secret Key");

if(!isset($_POST['DataType']))
	ExitDebug("Missing Data Type");

if(!isset($_POST['GameID']))
	ExitDebug("Missing GameID");	

if(!isset($_POST['Port']))
	ExitDebug("Missing Port");

if(!isset($_POST['CK']))
	ExitDebug("Missing CK");

if(!isset($_POST['Name']))
	ExitDebug("Missing Server Name");

if(!isset($_POST['Message']))
	ExitDebug("Missing Server Message");	

if(!isset($_POST['Type']))
	ExitDebug("Missing Server Type");

if(!isset($_POST['Country']))
	ExitDebug("Missing Server Country");

if(!isset($_POST['Mod']))
	ExitDebug("Missing Server Mod");

if(!isset($_POST['GameType']))
	ExitDebug("Missing Game Type");

if(!isset($_POST['CurrentPlayers']))
	ExitDebug("Missing Current Players");

if(!isset($_POST['MaxPlayers']))
	ExitDebug("Missing Max Players");

if(!isset($_POST['MissionName']))
	ExitDebug("Missing Mission Name");

if(!isset($_POST['MissionFile']))
	ExitDebug("Missing Mission File");	

if(!isset($_POST['TimeRemaining']))
	ExitDebug("Missing Time Remaining");

if(!isset($_POST['Password']))
	ExitDebug("Missing Server Password");

if(!isset($_POST['PlayerList']))
	ExitDebug("Missing Player List");

if(empty($_SERVER['REMOTE_ADDR']))
	ExitDebug("Missing IP Address");

//########################################################################
// Decode / Char Encoding
//########################################################################

if(!$_POST['PlayerList']=@base64_decode($_POST['PlayerList']))
	ExitDebug("Cannot decode player list");

foreach($_POST AS $Key=>$Val)
	$_POST[$Key] = iconv("WINDOWS-1252", "UTF-8", $_POST[$Key]);

//########################################################################
// Expectations
//########################################################################

if($_POST['SKey']!=$Secret_Key)
	ExitDebug("Invalid Secret Key");

if(!preg_match('/^[a-z0-9]+$/', $_POST['GameID']))
	ExitDebug("Invalid Game ID");

if(!empty($Expected_Game) && $Expected_Game!=$_POST['GameID'])
	ExitDebug("Invalid Expected Game ID");

if($_POST['DataType']!="0x100")
	ExitDebug("Invalid Data Type");

if(!preg_match('/^[0-9.]+$/', $_SERVER['REMOTE_ADDR']))
	ExitDebug("Invalid IP Address");

if(!preg_match('/^[0-9]+$/', $_POST["Port"]))
	ExitDebug("Invalid Port");

if(!preg_match('/^[0-9]+$/', $_POST["CK"]))
	ExitDebug("Invalid CK");

if($_POST["Type"]!="Dedicated" && $_POST["Type"]!="Non-Dedicated")
	ExitDebug("Invalid Server Type");

if(!preg_match('/^[A-Z]+$/', $_POST["GameType"]))
	ExitDebug("Invalid Game Type");

if(!preg_match('/^[0-9]+$/', $_POST["CurrentPlayers"]))
	ExitDebug("Invalid Current Players");

if(!preg_match('/^[0-9]+$/', $_POST["MaxPlayers"]))
	ExitDebug("Invalid Max Players");

if(!preg_match('/^[0-9-:]+$/', $_POST["TimeRemaining"]))
	ExitDebug("Invalid Time Remaining");

//########################################################################
// Update the database
//########################################################################

$Sql = "
	SELECT 
		`Name`
	FROM `".$MySql_Table."` 
	WHERE IPAddress='".$Db->real_escape_string($_SERVER['REMOTE_ADDR'])."' 
	AND Port='".intval($_POST['Port'])."' 
	AND GameID='".$Db->real_escape_string($_POST['GameID'])."' 
";

if(!$Result = $Db->query($Sql))
	ExitDebug("Database Error: ".$Db->error);

if($Result->num_rows===0) {

	$SqlUpdate="
		INSERT INTO `".$MySql_Table."` (
			`GameID`,
			`IPAddress`,
			`Port`,
			`CK`,
			`Name`,
			`Mod`,
			`Message`,
			`Type`,
			`Country`,
			`GameType`,
			`CurrentPlayers`,
			`MaxPlayers`,
			`MissionName`,
			`MissionFile`,
			`TimeRemaining`,
			`PlayerList`,
			`LastReport`
		) VALUES (
			'".$Db->real_escape_string($_POST['GameID'])."',
			'".$Db->real_escape_string($_SERVER['REMOTE_ADDR'])."',
			'".intval($_POST['Port'])."',
			'".intval($_POST['CK'])."',
			'".$Db->real_escape_string($_POST['Name'])."',
			'".$Db->real_escape_string($_POST['Mod'])."',
			'".$Db->real_escape_string($_POST['Message'])."',
			'".$Db->real_escape_string($_POST['Type'])."',
			'".$Db->real_escape_string($_POST['Country'])."',
			'".$Db->real_escape_string($_POST['GameType'])."',
			'".intval($_POST['CurrentPlayers'])."',
			'".intval($_POST['MaxPlayers'])."',
			'".$Db->real_escape_string($_POST['MissionName'])."',
			'".$Db->real_escape_string($_POST['MissionFile'])."',
			'".$Db->real_escape_string($_POST['TimeRemaining'])."',
			'".base64_encode($_POST['PlayerList'])."',
			NOW()
		)
	";

} else {

	$SqlUpdate="
		UPDATE  `".$MySql_Table."`  
		SET
			`CK`='".intval($_POST['CK'])."',
			`Name`='".$Db->real_escape_string($_POST['Name'])."',
			`Mod`='".$Db->real_escape_string($_POST['Mod'])."',
			`Message`='".$Db->real_escape_string($_POST['Message'])."',
			`Type`='".$Db->real_escape_string($_POST['Type'])."',
			`Country`='".$Db->real_escape_string($_POST['Country'])."',
			`GameType`='".$Db->real_escape_string($_POST['GameType'])."',
			`CurrentPlayers`='".intval($_POST['CurrentPlayers'])."',
			`MaxPlayers`='".intval($_POST['MaxPlayers'])."',
			`MissionName`='".$Db->real_escape_string($_POST['MissionName'])."',
			`MissionFile`='".$Db->real_escape_string($_POST['MissionFile'])."',
			`TimeRemaining`='".$Db->real_escape_string($_POST['TimeRemaining'])."',
			`PlayerList`='".base64_encode($_POST['PlayerList'])."',
			`LastReport`=NOW()
		WHERE `IPAddress`='".$Db->real_escape_string($_SERVER['REMOTE_ADDR'])."' 
		AND `Port`='".intval($_POST['Port'])."' 
		AND `GameID`='".$Db->real_escape_string($_POST['GameID'])."' 
	";

}

if(!$Db->query($SqlUpdate))
	ExitDebug("Database Error: ".$Db->error);

ExitDebug("Update was a success!");